"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startSaving = exports.initLogger = exports.errorLogger = exports.fatalLogger = exports.mainLogger = void 0;
const log4js = require("log4js");
const fs_1 = require("fs");
const log4jsExtend = require("log4js-extend");
const console_1 = require("console");
const stream_1 = require("stream");
const path_1 = require("path");
const streamroller_1 = require("streamroller");
const path_2 = require("path");
class Logger {
    constructor(baseDir, config) {
        this.baseDir = baseDir;
        this.config = config;
        this.logDir = '';
        this.log4Conf = null;
        this._setupLogs();
    }
    // Checks if the configuration has the required components
    _checkValidConfig() {
        const config = this.config;
        if (!config.dir)
            throw Error('Fatal Error: Log directory not defined.');
        if (!config.files || typeof config.files !== 'object')
            throw Error('Fatal Error: Valid log file locations not provided.');
    }
    // Add filenames to each appender of type 'file'
    _addFileNamesToAppenders() {
        const conf = this.log4Conf;
        for (const key in conf.appenders) {
            // ignoring due to not coming from user input or exxternal source therefore unlikely this would lead to a prototype pollution attack
            // eslint-disable-next-line security/detect-object-injection
            const appender = conf.appenders[key];
            if (appender.type !== 'file')
                continue;
            appender.filename = `${this.logDir}/${key}.log`;
        }
    }
    _configureLogs() {
        return log4js.configure(this.log4Conf);
    }
    // Get the specified logger
    getLogger(logger) {
        return log4js.getLogger(logger);
    }
    // Setup the logs with the provided configuration using the base directory provided for relative paths
    _setupLogs() {
        const baseDir = this.baseDir;
        const config = this.config;
        if (!baseDir)
            throw Error('Fatal Error: Base directory not defined.');
        if (!config)
            throw Error('Fatal Error: No configuration provided.');
        this._checkValidConfig();
        // Makes specified directory if it doesn't exist
        if (config.dir) {
            // normalize and resolve the path to avoid path traversal attacks
            const allArchiversLogDir = (0, path_2.resolve)((0, path_2.normalize)(`${baseDir}/${config.dir.split('/')[0]}`));
            this.getLogger('main').info('allArchiversLogDir', allArchiversLogDir);
            // eslint-disable-next-line security/detect-non-literal-fs-filename
            if (!(0, fs_1.existsSync)(allArchiversLogDir))
                (0, fs_1.mkdirSync)(allArchiversLogDir);
        }
        // normalize and resolve the path to avoid path traversal attacks
        this.logDir = (0, path_2.resolve)((0, path_2.normalize)(`${baseDir}/${config.dir}`));
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        if (!(0, fs_1.existsSync)(this.logDir))
            (0, fs_1.mkdirSync)(this.logDir);
        // Read the log config from log config file
        this.log4Conf = config.options;
        log4jsExtend(log4js);
        this._addFileNamesToAppenders();
        this._configureLogs();
        this.getLogger('main').info('Logger initialized.');
    }
    // Tells this module that the server is shutting down, returns a Promise that resolves when all logs have been written to file, sockets are closed, etc.
    shutdown() {
        return new Promise((resolve) => {
            log4js.shutdown(() => {
                resolve('done');
            });
        });
    }
}
function initLogger(baseDir, logsConfig) {
    const logger = new Logger(baseDir, logsConfig);
    exports.mainLogger = logger.getLogger('main');
    exports.fatalLogger = logger.getLogger('fatal');
    exports.errorLogger = logger.getLogger('errorFile');
    // Start saving console output to file
    startSaving((0, path_1.join)(baseDir, logsConfig.dir));
}
exports.initLogger = initLogger;
exports.default = Logger;
function startSaving(baseDir) {
    // Create a file to save combined stdout and stderr output
    const outFileName = `out.log`;
    const stream = new streamroller_1.RollingFileStream((0, path_1.join)(baseDir, outFileName), 10000000, 10);
    // Create passthroughs that write to stdout, stderr, and the output file
    const outPass = new stream_1.PassThrough();
    outPass.pipe(process.stdout);
    outPass.pipe(stream);
    const errPass = new stream_1.PassThrough();
    errPass.pipe(process.stderr);
    errPass.pipe(stream);
    // Monkey patch the global console with a new one that uses our passthroughs
    console = new console_1.Console({ stdout: outPass, stderr: errPass }); // eslint-disable-line no-global-assign
}
exports.startSaving = startSaving;
//# sourceMappingURL=Logger.js.map