"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateRequestData = exports.registerRoutes = exports.MAX_BETWEEN_CYCLES_PER_REQUEST = exports.MAX_CYCLES_PER_REQUEST = exports.MAX_ORIGINAL_TXS_PER_REQUEST = exports.MAX_RECEIPTS_PER_REQUEST = exports.MAX_ACCOUNTS_PER_REQUEST = void 0;
const Config_1 = require("./Config");
const Logger = require("./Logger");
const Crypto = require("./utils/Crypto");
const Utils = require("./utils/Utils");
const CycleDB = require("./dbstore/cycles");
const AccountDB = require("./dbstore/accounts");
const TransactionDB = require("./dbstore/transactions");
const ReceiptDB = require("./dbstore/receipts");
const OriginalTxDB = require("./dbstore/originalTxsData");
const utils_1 = require("./distributor/utils");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const metrics_1 = require("./metrics");
const TXID_LENGTH = 64;
exports.MAX_ACCOUNTS_PER_REQUEST = 1000;
exports.MAX_RECEIPTS_PER_REQUEST = 100;
exports.MAX_ORIGINAL_TXS_PER_REQUEST = 100;
exports.MAX_CYCLES_PER_REQUEST = 100;
exports.MAX_BETWEEN_CYCLES_PER_REQUEST = 100;
function registerRoutes(server) {
    server.post('/cycleinfo', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            start: 'n?',
            end: 'n?',
            count: 'n?',
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        const { start, end, count } = requestData;
        let cycleInfo = [];
        if (count) {
            if (count <= 0 || Number.isNaN(count)) {
                reply.send(Crypto.sign({ success: false, error: `Invalid count` }));
                return;
            }
            if (count > exports.MAX_CYCLES_PER_REQUEST) {
                reply.send(Crypto.sign({ success: false, error: `Max count is ${exports.MAX_CYCLES_PER_REQUEST}.` }));
                return;
            }
            cycleInfo = (await CycleDB.queryLatestCycleRecords(count)) || [];
        }
        else if (start || start === 0) {
            const from = start;
            const to = end ? end : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                Logger.mainLogger.error(`Invalid start and end counters`);
                reply.send(Crypto.sign({ success: false, error: `Invalid start and end counters` }));
                return;
            }
            const cycleCount = to - from;
            if (cycleCount > exports.MAX_CYCLES_PER_REQUEST) {
                Logger.mainLogger.error(`Exceed maximum limit of ${exports.MAX_CYCLES_PER_REQUEST} cycles`);
                reply.send(Crypto.sign({ success: false, error: `Exceed maximum limit of ${exports.MAX_CYCLES_PER_REQUEST} cycles` }));
                return;
            }
            cycleInfo = (await CycleDB.queryCycleRecordsBetween(from, to)) || [];
        }
        else {
            reply.send({
                success: false,
                error: 'not specified which cycle to show',
            });
            return;
        }
        const res = Crypto.sign({
            cycleInfo,
        });
        reply.send(res);
    });
    server.post('/originalTx', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            count: 'n?',
            start: 'n?',
            end: 'n?',
            startCycle: 'n?',
            endCycle: 'n?',
            type: 's?',
            page: 'n?',
            txId: 's?',
            txIdList: 's?',
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        const { count, start, end, startCycle, endCycle, type, page, txId, txIdList } = requestData;
        let originalTxs = [];
        if (count) {
            if (count <= 0 || Number.isNaN(count)) {
                reply.send(Crypto.sign({ success: false, error: `Invalid count` }));
                return;
            }
            if (count > exports.MAX_ORIGINAL_TXS_PER_REQUEST) {
                reply.send(Crypto.sign({ success: false, error: `Max count is ${exports.MAX_ORIGINAL_TXS_PER_REQUEST}` }));
                return;
            }
            originalTxs = (await OriginalTxDB.queryLatestOriginalTxs(count)) || [];
        }
        else if (txId) {
            if (txId.length !== TXID_LENGTH) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid txId ${txId}`,
                }));
                return;
            }
            const originalTx = await OriginalTxDB.queryOriginalTxDataByTxId(txId);
            if (originalTx)
                originalTxs.push(originalTx);
        }
        else if (txIdList) {
            let txIdListArr = [];
            try {
                txIdListArr = lib_types_1.Utils.safeJsonParse(txIdList);
                if (txIdListArr && txIdListArr.length > exports.MAX_ORIGINAL_TXS_PER_REQUEST) {
                    reply.send(Crypto.sign({
                        success: false,
                        error: `Exceed maximum limit of ${exports.MAX_ORIGINAL_TXS_PER_REQUEST} original transactions`,
                    }));
                    return;
                }
            }
            catch (e) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid txIdList ${txIdList}`,
                }));
                return;
            }
            for (const txId of txIdListArr) {
                if (typeof txId !== 'string' || txId.length !== TXID_LENGTH) {
                    reply.send(Crypto.sign({
                        success: false,
                        error: `Invalid txId ${txId} in the List`,
                    }));
                    return;
                }
                const originalTx = (await OriginalTxDB.queryOriginalTxDataByTxId(txId));
                if (originalTx)
                    originalTxs.push(originalTx);
            }
        }
        else if (start || start === 0) {
            const from = start;
            const to = end ? end : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid start and end counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_ORIGINAL_TXS_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_ORIGINAL_TXS_PER_REQUEST} original transactions`,
                }));
                return;
            }
            originalTxs = await OriginalTxDB.queryOriginalTxsData(from, count + 1);
        }
        else if (startCycle || startCycle === 0) {
            const from = startCycle;
            const to = endCycle ? endCycle : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid startCycle and endCycle counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_BETWEEN_CYCLES_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_BETWEEN_CYCLES_PER_REQUEST} cycles`,
                }));
                return;
            }
            if (type === 'tally') {
                originalTxs = await OriginalTxDB.queryOriginalTxDataCountByCycles(from, to);
            }
            else if (type === 'count') {
                originalTxs = await OriginalTxDB.queryOriginalTxDataCount(from, to);
            }
            else {
                let skip = 0;
                const limit = exports.MAX_ORIGINAL_TXS_PER_REQUEST;
                if (page) {
                    if (page < 1 || Number.isNaN(page)) {
                        reply.send(Crypto.sign({ success: false, error: `Invalid page number` }));
                        return;
                    }
                    skip = page - 1;
                    if (skip > 0)
                        skip = skip * limit;
                }
                originalTxs = await OriginalTxDB.queryOriginalTxsData(skip, limit, from, to);
            }
        }
        const res = Crypto.sign({
            originalTxs,
        });
        reply.send(res);
    });
    server.post('/receipt', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            count: 'n?',
            start: 'n?',
            end: 'n?',
            startCycle: 'n?',
            endCycle: 'n?',
            type: 's?',
            page: 'n?',
            txId: 's?',
            txIdList: 's?',
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        const { count, start, end, startCycle, endCycle, type, page, txId, txIdList } = requestData;
        let receipts = [];
        if (count) {
            if (count <= 0 || Number.isNaN(count)) {
                reply.send(Crypto.sign({ success: false, error: `Invalid count` }));
                return;
            }
            if (count > exports.MAX_RECEIPTS_PER_REQUEST) {
                reply.send(Crypto.sign({ success: false, error: `Max count is ${exports.MAX_RECEIPTS_PER_REQUEST}` }));
                return;
            }
            receipts = (await ReceiptDB.queryLatestReceipts(count)) || [];
        }
        else if (txId) {
            if (txId.length !== TXID_LENGTH) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid txId ${txId}`,
                }));
                return;
            }
            const receipt = await ReceiptDB.queryReceiptByReceiptId(txId);
            if (receipt)
                receipts.push(receipt);
        }
        else if (txIdList) {
            let txIdListArr = [];
            try {
                txIdListArr = lib_types_1.Utils.safeJsonParse(txIdList);
                if (txIdListArr && txIdListArr.length > exports.MAX_RECEIPTS_PER_REQUEST) {
                    reply.send(Crypto.sign({
                        success: false,
                        error: `Exceed maximum limit of ${exports.MAX_RECEIPTS_PER_REQUEST} receipts`,
                    }));
                    return;
                }
            }
            catch (e) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid txIdList ${txIdList}`,
                }));
                return;
            }
            for (const txId of txIdListArr) {
                if (typeof txId !== 'string' || txId.length !== TXID_LENGTH) {
                    reply.send(Crypto.sign({
                        success: false,
                        error: `Invalid txId ${txId} in the List`,
                    }));
                    return;
                }
                const receipt = await ReceiptDB.queryReceiptByReceiptId(txId);
                if (receipt)
                    receipts.push(receipt);
            }
        }
        else if (start || start === 0) {
            const from = start;
            const to = end ? end : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid start and end counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_RECEIPTS_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_RECEIPTS_PER_REQUEST} receipts`,
                }));
                return;
            }
            receipts = (await ReceiptDB.queryReceipts(from, count + 1)) || [];
        }
        else if (startCycle || startCycle === 0) {
            const from = startCycle;
            const to = endCycle ? endCycle : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid startCycle and endCycle counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_BETWEEN_CYCLES_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_BETWEEN_CYCLES_PER_REQUEST} cycles`,
                }));
                return;
            }
            if (type === 'tally') {
                receipts = (await ReceiptDB.queryReceiptCountByCycles(from, to)) || [];
            }
            else if (type === 'count') {
                receipts = await ReceiptDB.queryReceiptCountBetweenCycles(from, to);
            }
            else {
                let skip = 0;
                const limit = exports.MAX_RECEIPTS_PER_REQUEST;
                if (page) {
                    if (page < 1 || Number.isNaN(page)) {
                        reply.send(Crypto.sign({ success: false, error: `Invalid page number` }));
                        return;
                    }
                    skip = page - 1;
                    if (skip > 0)
                        skip = skip * limit;
                }
                receipts = await ReceiptDB.queryReceiptsBetweenCycles(skip, limit, from, to);
            }
        }
        const res = Crypto.sign({
            receipts,
        });
        reply.send(res);
    });
    server.post('/account', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            count: 'n?',
            start: 'n?',
            end: 'n?',
            startCycle: 'n?',
            endCycle: 'n?',
            page: 'n?',
            accountId: 's?',
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        let accounts = [];
        let totalAccounts = 0;
        let res;
        const { count, start, end, startCycle, endCycle, page, accountId } = requestData;
        if (count) {
            if (count <= 0 || Number.isNaN(count)) {
                reply.send(Crypto.sign({ success: false, error: `Invalid count` }));
                return;
            }
            if (count > exports.MAX_ACCOUNTS_PER_REQUEST) {
                reply.send(Crypto.sign({ success: false, error: `Max count is ${exports.MAX_ACCOUNTS_PER_REQUEST}` }));
                return;
            }
            accounts = (await AccountDB.queryLatestAccounts(count)) || [];
        }
        else if (start || start === 0) {
            const from = start;
            const to = end ? end : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid start and end counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_ACCOUNTS_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_ACCOUNTS_PER_REQUEST} accounts`,
                }));
                return;
            }
            accounts = (await AccountDB.queryAccounts(from, count + 1)) || [];
            res = Crypto.sign({
                accounts,
            });
        }
        else if (startCycle || startCycle === 0) {
            const from = startCycle;
            const to = endCycle ? endCycle : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid startCycle and endCycle counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_BETWEEN_CYCLES_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_BETWEEN_CYCLES_PER_REQUEST} cycles to query accounts Count`,
                }));
                return;
            }
            totalAccounts = (await AccountDB.queryAccountCountBetweenCycles(from, to)) || 0;
            if (page) {
                if (page < 1 || Number.isNaN(page)) {
                    reply.send(Crypto.sign({ success: false, error: `Invalid page number` }));
                    return;
                }
                let skip = page - 1;
                const limit = exports.MAX_ACCOUNTS_PER_REQUEST;
                if (skip > 0)
                    skip = skip * limit;
                accounts = (await AccountDB.queryAccountsBetweenCycles(skip, limit, from, to)) || [];
                res = Crypto.sign({
                    accounts,
                    totalAccounts,
                });
            }
            else {
                res = Crypto.sign({
                    totalAccounts,
                });
            }
        }
        else if (accountId) {
            const account = await AccountDB.queryAccountByAccountId(accountId);
            accounts = account ? [account] : [];
            res = Crypto.sign({
                accounts,
            });
        }
        else {
            reply.send({
                success: false,
                error: 'not specified which account to show',
            });
            return;
        }
        reply.send(res);
    });
    server.post('/transaction', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            count: 'n?',
            start: 'n?',
            end: 'n?',
            txId: 's?',
            appReceiptId: 's?',
            startCycle: 'n?',
            endCycle: 'n?',
            page: 'n?',
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        const { count, start, end, txId, appReceiptId, startCycle, endCycle, page } = requestData;
        let transactions = [];
        let totalTransactions = 0;
        let res;
        if (count) {
            if (count <= 0 || Number.isNaN(count)) {
                reply.send(Crypto.sign({ success: false, error: `Invalid count` }));
                return;
            }
            if (count > exports.MAX_ACCOUNTS_PER_REQUEST) {
                reply.send(Crypto.sign({ success: false, error: `Max count is ${exports.MAX_ACCOUNTS_PER_REQUEST}` }));
                return;
            }
            transactions = (await TransactionDB.queryLatestTransactions(count)) || [];
        }
        else if (start || start === 0) {
            const from = start;
            const to = end ? end : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid start and end counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_ACCOUNTS_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_ACCOUNTS_PER_REQUEST} transactions`,
                }));
                return;
            }
            transactions = (await TransactionDB.queryTransactions(from, count + 1)) || [];
            res = Crypto.sign({
                transactions,
            });
        }
        else if (startCycle || startCycle === 0) {
            const from = startCycle;
            const to = endCycle ? endCycle : from;
            if (!(from >= 0 && to >= from) || Number.isNaN(from) || Number.isNaN(to)) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Invalid startCycle and endCycle counters`,
                }));
                return;
            }
            const count = to - from;
            if (count > exports.MAX_BETWEEN_CYCLES_PER_REQUEST) {
                reply.send(Crypto.sign({
                    success: false,
                    error: `Exceed maximum limit of ${exports.MAX_BETWEEN_CYCLES_PER_REQUEST} cycles to query transactions Count`,
                }));
                return;
            }
            totalTransactions = await TransactionDB.queryTransactionCountBetweenCycles(from, to);
            if (page) {
                if (page < 1 || Number.isNaN(page)) {
                    reply.send(Crypto.sign({ success: false, error: `Invalid page number` }));
                    return;
                }
                let skip = page - 1;
                const limit = exports.MAX_ACCOUNTS_PER_REQUEST;
                if (skip > 0)
                    skip = skip * limit;
                transactions = (await TransactionDB.queryTransactionsBetweenCycles(skip, limit, from, to)) || [];
                res = Crypto.sign({
                    transactions,
                    totalTransactions,
                });
            }
            else {
                res = Crypto.sign({
                    totalTransactions,
                });
            }
        }
        else if (txId) {
            const transaction = await TransactionDB.queryTransactionByTxId(txId);
            transactions = transaction ? [transaction] : [];
            res = Crypto.sign({
                transactions,
            });
        }
        else if (appReceiptId) {
            const transaction = await TransactionDB.queryTransactionByAccountId(appReceiptId);
            transactions = transaction ? [transaction] : [];
            res = Crypto.sign({
                transactions,
            });
        }
        else {
            res = {
                success: false,
                error: 'not specified which account to show',
            };
        }
        reply.send(res);
    });
    server.post('/totalData', async (_request, reply) => {
        const requestData = _request.body;
        const result = (0, exports.validateRequestData)(requestData, {
            sender: 's',
            sign: 'o',
        });
        if (!result.success) {
            reply.send(Crypto.sign({ success: false, error: result.error }));
            return;
        }
        const totalCycles = await CycleDB.queryCyleCount();
        const totalAccounts = await AccountDB.queryAccountCount();
        const totalTransactions = await TransactionDB.queryTransactionCount();
        const totalReceipts = await ReceiptDB.queryReceiptCount();
        const totalOriginalTxs = await OriginalTxDB.queryOriginalTxDataCount();
        reply.send(Crypto.sign({
            totalCycles,
            totalAccounts,
            totalTransactions,
            totalReceipts,
            totalOriginalTxs,
            pid: process.pid,
        }));
    });
    server.get('/config', (_request, reply) => {
        Config_1.config.DISTRIBUTOR_PUBLIC_KEY;
        const distributorConfig = {
            DISTRIBUTOR_IP: Config_1.config.DISTRIBUTOR_IP,
            DISTRIBUTOR_PORT: Config_1.config.DISTRIBUTOR_PORT,
            DISTRIBUTOR_PUBLIC_KEY: Config_1.config.DISTRIBUTOR_PUBLIC_KEY,
            limitToSubscribersOnly: Config_1.config.limitToSubscribersOnly,
            subscribers: Config_1.config.subscribers,
        };
        reply.send({ config: distributorConfig });
    });
    server.get('/metrics', (_request, reply) => {
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Metrics requested');
        }
        try {
            const metrics = (0, metrics_1.getDataLogReaderMetrics)();
            if (Config_1.config.VERBOSE) {
                Logger.mainLogger.debug('Metrics: ', metrics);
            }
            reply.send(metrics);
        }
        catch (e) {
            Logger.mainLogger.error('Error getting metrics', e);
            reply.status(500).send({ error: 'Error getting metrics' });
        }
    });
}
exports.registerRoutes = registerRoutes;
const validateRequestData = (data, expectedDataType) => {
    try {
        let err = Utils.validateTypes(data, expectedDataType);
        if (err) {
            Logger.mainLogger.error('Invalid request data ', err);
            return { success: false, error: 'Invalid request data ' + err };
        }
        err = Utils.validateTypes(data.sign, { owner: 's', sig: 's' });
        if (err) {
            Logger.mainLogger.error('Invalid request data signature ', err);
            return { success: false, error: 'Invalid request data signature ' + err };
        }
        if (data.sign.owner !== data.sender) {
            Logger.mainLogger.error('Data sender publicKey and sign owner key does not match');
            return { success: false, error: 'Data sender publicKey and sign owner key does not match' };
        }
        if (Config_1.config.limitToSubscribersOnly) {
            if (!utils_1.distributorSubscribers.has(data.sender)) {
                Logger.mainLogger.error('Data request sender is not a subscriber');
                return { success: false, error: 'Data request sender is not a subscriber' };
            }
            const subscriber = utils_1.distributorSubscribers.get(data.sender);
            if (subscriber?.expirationTimestamp !== 0 && subscriber.expirationTimestamp > Date.now()) {
                Logger.mainLogger.error('Subscriber subscription expired');
                return { success: false, error: 'Subscriber subscription expired' };
            }
            if (expectedDataType.collectorInfo) {
                err = Utils.validateTypes(data.collectorInfo, { subscriptionType: 's', timestamp: 'n' });
                if (err) {
                    Logger.mainLogger.error('Invalid collectorInfo ', err);
                    return { success: false, error: 'Invalid collectorInfo ' + err };
                }
                if (data.collectorInfo.subscriptionType !== subscriber.subscriptionType) {
                    Logger.mainLogger.error('Invalid subscriptionType');
                    return { success: false, error: 'Invalid subscriptionType' };
                }
                // Check if the timestamp is less than 30 seconds of the current time
                const ACCEPTABLE_TIMESTAMP_DIFF = 30000;
                if (data.collectorInfo.timestamp !== 0 &&
                    Date.now() - data.collectorInfo.timestamp > ACCEPTABLE_TIMESTAMP_DIFF) {
                    Logger.mainLogger.error('Invalid timestamp');
                    return { success: false, error: 'Invalid timestamp' };
                }
            }
        }
        if (!Crypto.verify(data)) {
            Logger.mainLogger.error('Invalid signature', data);
            return { success: false, error: 'Invalid signature' };
        }
        return { success: true };
    }
    catch (e) {
        Logger.mainLogger.error('Error validating request data', e);
        return { success: false, error: 'Error validating request data' };
    }
};
exports.validateRequestData = validateRequestData;
//# sourceMappingURL=api.js.map