"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerDataReaderListeners = exports.registerParentProcessListener = exports.handleSocketRequest = void 0;
const WebSocket = require("ws");
const Crypto = require("../utils/Crypto");
const path_1 = require("path");
const Config_1 = require("../Config");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const metrics_1 = require("../metrics");
const FILE = (0, path_1.join)(process.cwd(), 'distributor-config.json');
(0, Config_1.overrideDefaultConfig)(FILE, process.env, process.argv);
Crypto.setCryptoHashKey(Config_1.config.DISTRIBUTOR_HASH_KEY);
var SocketCloseCodes;
(function (SocketCloseCodes) {
    SocketCloseCodes[SocketCloseCodes["DUPLICATE_CONNECTION_CODE"] = 1000] = "DUPLICATE_CONNECTION_CODE";
    SocketCloseCodes[SocketCloseCodes["SUBSCRIBER_EXPIRATION_CODE"] = 1001] = "SUBSCRIBER_EXPIRATION_CODE";
})(SocketCloseCodes || (SocketCloseCodes = {}));
const wss = new WebSocket.Server({ noServer: true });
const socketClientMap = new Map();
const handleSocketRequest = (dataProp) => {
    if (dataProp.headers) {
        wss.handleUpgrade(dataProp, dataProp.socket, dataProp.head, (ws) => {
            const clientId = dataProp.clientKey;
            if (socketClientMap.has(clientId)) {
                console.info('Duplicate Connection Found. Closing previous connection.');
                socketClientMap.get(clientId).close(1000);
            }
            socketClientMap.set(clientId, ws);
            // Sending Client-ID to Parent Process for managing subscribers
            process.send({
                type: 'client_connected',
                data: clientId,
            });
            ws.on('close', (code) => {
                switch (code) {
                    case SocketCloseCodes.DUPLICATE_CONNECTION_CODE:
                        console.log(`❌ Connection with Duplicate Client (${clientId}) Terminated.`);
                        // Note: Since this event is triggered originally from the parent process, we don't need to send a message to the parent process
                        break;
                    case SocketCloseCodes.SUBSCRIBER_EXPIRATION_CODE:
                        console.log(`❌ Expired Subscriber (${clientId}) Closed.`);
                        process.send({
                            type: 'client_expired',
                            data: clientId,
                        });
                        break;
                    default:
                        process.send({
                            type: 'client_close',
                            data: clientId,
                        });
                        console.log(`❌ Connection with Client (${clientId}) Closed.`);
                }
                if (socketClientMap.has(clientId))
                    socketClientMap.delete(clientId);
            });
        });
    }
};
exports.handleSocketRequest = handleSocketRequest;
const registerParentProcessListener = () => {
    process.on('message', (dataProp) => {
        const clientId = dataProp.data;
        switch (dataProp.type) {
            case 'remove_subscriber':
                socketClientMap.get(clientId).close(1001);
                console.log(`❌ Expired Subscription Client (${clientId}) Removed.`);
                break;
            case 'close_duplicate_connection':
                socketClientMap.get(clientId).close(1000);
                console.log(`❌ Duplicate Client Connection Removed.`);
                break;
        }
    });
};
exports.registerParentProcessListener = registerParentProcessListener;
const sendDataToAllClients = ({ signedData }) => {
    for (const client of socketClientMap.values()) {
        client.send(lib_types_1.Utils.safeStringify({
            ...signedData,
        }));
    }
};
const registerDataReaderListeners = (reader) => {
    reader.on(`${reader.dataName}-data`, (logData) => {
        try {
            const data = {};
            switch (reader.dataName) {
                case 'cycle':
                    data.cycle = logData;
                    (0, metrics_1.incrementCycleCount)();
                    break;
                case 'receipt':
                    data.receipt = logData;
                    (0, metrics_1.incrementReceiptCount)();
                    break;
                case 'originalTx':
                    data.originalTx = logData;
                    (0, metrics_1.incrementOriginalTxCount)();
                    break;
            }
            sendDataToAllClients({
                signedData: Crypto.sign(data, Config_1.config.DISTRIBUTOR_SECRET_KEY, Config_1.config.DISTRIBUTOR_PUBLIC_KEY),
            });
        }
        catch (e) {
            console.log('Issue with Log-reader data: ');
            console.log('->> LOG DATA: ', logData);
            console.log(e);
        }
    });
    reader.on(`${reader.dataName}-end`, (totalEntriesItReads, totalEntriesDefinedOnFile) => {
        console.info(`✔️ Finished reading ${totalEntriesItReads} entries from ${reader.dataName}-${reader.logCounter} log file having ${totalEntriesDefinedOnFile} entries.`);
    });
    reader.on('error', (err) => {
        console.error(`Error reading log file: ${err}`);
    });
};
exports.registerDataReaderListeners = registerDataReaderListeners;
process.on('uncaughtException', (error) => {
    console.error('Uncaught Exception in Child Process:', error);
});
//# sourceMappingURL=child.js.map