"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initWorker = exports.initHttpServer = exports.workerClientMap = void 0;
const url = require("url");
const http = require("http");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const log_reader_1 = require("../log-reader");
const cors_1 = require("@fastify/cors");
const child_1 = require("./child");
const fastify_1 = require("fastify");
const rate_limit_1 = require("@fastify/rate-limit");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const api_1 = require("../api");
const healthCheck_1 = require("../routes/healthCheck");
let httpServer;
exports.workerClientMap = new Map();
const connectToSocketClient = (clientKey, clientRequestData) => {
    try {
        (0, child_1.handleSocketRequest)({ ...clientRequestData.header, socket: clientRequestData.socket, clientKey });
    }
    catch (e) {
        throw new Error(`Error in connectToSocketClient(): ${e}`);
    }
};
const initHttpServer = async (worker) => {
    const serverFactory = (handler) => {
        httpServer = http.createServer((req, res) => {
            handler(req, res);
        });
        return httpServer;
    };
    const fastifyServer = (0, fastify_1.default)({ serverFactory });
    await fastifyServer.register(cors_1.default);
    await fastifyServer.register(rate_limit_1.default, {
        global: true,
        max: Config_1.config.RATE_LIMIT,
        timeWindow: 10,
        allowList: ['127.0.0.1', '0.0.0.0'], // Excludes local IPs from rate limits
    });
    await fastifyServer.register(healthCheck_1.healthCheckRouter);
    fastifyServer.addContentTypeParser('application/json', { parseAs: 'string' }, (req, body, done) => {
        try {
            const jsonString = typeof body === 'string' ? body : body.toString('utf8');
            done(null, lib_types_1.Utils.safeJsonParse(jsonString));
        }
        catch (err) {
            err.statusCode = 400;
            done(err, undefined);
        }
    });
    fastifyServer.setReplySerializer((payload) => {
        return lib_types_1.Utils.safeStringify(payload);
    });
    // Register API routes
    (0, api_1.registerRoutes)(fastifyServer);
    initSocketServer(httpServer, worker);
    (0, child_1.registerParentProcessListener)();
    // Start server and bind to port on all interfaces
    fastifyServer.ready(() => {
        httpServer.listen(Config_1.config.DISTRIBUTOR_PORT, () => {
            console.log(`Distributor-Server (${process.pid}) listening on port ${Config_1.config.DISTRIBUTOR_PORT}.`);
            Logger.mainLogger.debug(`API-Server started on Worker Process (${process.pid}).`);
            return;
        });
        httpServer.on('error', (err) => {
            Logger.mainLogger.error('Distributor failed to start.', err);
            process.exit(1);
        });
    });
};
exports.initHttpServer = initHttpServer;
const initSocketServer = async (httpServer, worker) => {
    // Handles incoming upgrade requests from clients (to upgrade to a Socket connection)
    httpServer.on('upgrade', (req, socket, head) => {
        const queryObject = url.parse(req.url, true).query;
        const decodedData = decodeURIComponent(queryObject.data);
        const clientData = lib_types_1.Utils.safeJsonParse(decodedData);
        const auth = (0, api_1.validateRequestData)(clientData, {
            collectorInfo: 'o',
            sender: 's',
            sign: 'o',
        });
        if (auth.success) {
            const clientKey = clientData.sender;
            connectToSocketClient(clientKey, {
                header: { headers: req.headers, method: req.method, head, clientKey },
                socket,
            });
        }
        else {
            Logger.mainLogger.debug(`❌ Unauthorized Client Request from ${req.headers.host}, Reason: ${auth.error}`);
            socket.write('HTTP/1.1 401 Unauthorized\r\n');
            socket.write('Content-Type: text/plain\r\n');
            socket.write('Connection: close\r\n');
            socket.write('Unauthorized: Authentication failed\r\n');
            socket.end();
            return;
        }
    });
};
const initWorker = async () => {
    try {
        if (Config_1.config.distributorMode === Config_1.distributorMode.MQ) {
            return;
        }
        const DATA_LOG_PATH = Config_1.config.DATA_LOG_DIR;
        const cycleReader = new log_reader_1.default(DATA_LOG_PATH, 'cycle');
        const receiptReader = new log_reader_1.default(DATA_LOG_PATH, 'receipt');
        const originalTxReader = new log_reader_1.default(DATA_LOG_PATH, 'originalTx');
        await Promise.all([receiptReader.init(), cycleReader.init(), originalTxReader.init()]);
        (0, child_1.registerDataReaderListeners)(cycleReader);
        (0, child_1.registerDataReaderListeners)(receiptReader);
        (0, child_1.registerDataReaderListeners)(originalTxReader);
    }
    catch (e) {
        if (e.code === 'ENOENT') {
            console.error('❌ Path to the data-logs directory does not exist. Please check the path in the config file.\n Current Path: ', Config_1.config.DATA_LOG_DIR);
            process.exit(0);
        }
        else {
            console.error('Error in Child Process: ', e.message, e.code);
        }
    }
};
exports.initWorker = initWorker;
//# sourceMappingURL=index.js.map