"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryAccountsBetweenCycles = exports.queryAccountCountBetweenCycles = exports.queryAccountCount = exports.queryAccounts = exports.queryLatestAccounts = exports.queryAccountByAccountId = exports.updateAccount = exports.bulkInsertAccounts = exports.insertAccount = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
async function insertAccount(account) {
    try {
        const fields = Object.keys(account).join(', ');
        const placeholders = Object.keys(account).fill('?').join(', ');
        const values = db.extractValues(account);
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from account ${account.accountId}`);
        }
        const sql = 'INSERT OR REPLACE INTO accounts (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(_1.accountDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted Account', account.accountId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert Account or it is already stored in to database', account.accountId);
    }
}
exports.insertAccount = insertAccount;
async function bulkInsertAccounts(accounts) {
    try {
        const fields = Object.keys(accounts[0]).join(', ');
        const placeholders = Object.keys(accounts[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(accounts);
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from accounts. Number of accounts: ${accounts.length}`);
        }
        let sql = 'INSERT OR REPLACE INTO accounts (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < accounts.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(_1.accountDatabase, sql, values);
        Logger.mainLogger.debug('Successfully inserted Accounts', accounts.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert Accounts', accounts.length);
    }
}
exports.bulkInsertAccounts = bulkInsertAccounts;
async function updateAccount(accountId, account) {
    try {
        const sql = `UPDATE accounts SET cycleNumber = $cycleNumber, timestamp = $timestamp, data = $data, hash = $hash WHERE accountId = $accountId `;
        await db.run(_1.accountDatabase, sql, {
            $cycleNumber: account.cycleNumber,
            $timestamp: account.timestamp,
            $data: account.data && (0, serialization_1.SerializeToJsonString)(account.data),
            $hash: account.hash,
            $accountId: account.accountId,
        });
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully updated Account', account.accountId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to update Account', account);
    }
}
exports.updateAccount = updateAccount;
async function queryAccountByAccountId(accountId) {
    try {
        const sql = `SELECT * FROM accounts WHERE accountId=?`;
        const account = (await db.get(_1.accountDatabase, sql, [accountId]));
        if (account)
            if (account && account.data)
                account.data = (0, serialization_1.DeSerializeFromJsonString)(account.data);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Account accountId', account);
        }
        return account;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryAccountByAccountId = queryAccountByAccountId;
async function queryLatestAccounts(count) {
    try {
        const sql = `SELECT * FROM accounts ORDER BY cycleNumber DESC, timestamp DESC LIMIT ${count ? count : 100}`;
        const accounts = (await db.all(_1.accountDatabase, sql));
        if (accounts.length > 0) {
            accounts.forEach((account) => {
                if (account && account.data)
                    account.data = (0, serialization_1.DeSerializeFromJsonString)(account.data);
            });
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Account latest', accounts);
        }
        return accounts;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryLatestAccounts = queryLatestAccounts;
async function queryAccounts(skip = 0, limit = 10000) {
    let accounts;
    try {
        const sql = `SELECT * FROM accounts ORDER BY cycleNumber ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        accounts = await db.all(_1.accountDatabase, sql);
        if (accounts.length > 0) {
            accounts.forEach((account) => {
                if (account && account.data)
                    account.data = (0, serialization_1.DeSerializeFromJsonString)(account.data);
            });
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Account accounts', accounts ? accounts.length : accounts, 'skip', skip);
    }
    return accounts;
}
exports.queryAccounts = queryAccounts;
async function queryAccountCount() {
    let accounts;
    try {
        const sql = `SELECT COUNT(*) FROM accounts`;
        accounts = await db.get(_1.accountDatabase, sql, []);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Account count', accounts);
    }
    if (accounts)
        accounts = accounts['COUNT(*)'];
    else
        accounts = 0;
    return accounts;
}
exports.queryAccountCount = queryAccountCount;
async function queryAccountCountBetweenCycles(startCycleNumber, endCycleNumber) {
    let accounts;
    try {
        const sql = `SELECT COUNT(*) FROM accounts WHERE cycleNumber BETWEEN ? AND ?`;
        accounts = await db.get(_1.accountDatabase, sql, [startCycleNumber, endCycleNumber]);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Account count between cycles', accounts);
    }
    if (accounts)
        accounts = accounts['COUNT(*)'];
    else
        accounts = 0;
    return accounts;
}
exports.queryAccountCountBetweenCycles = queryAccountCountBetweenCycles;
async function queryAccountsBetweenCycles(skip = 0, limit = 10000, startCycleNumber, endCycleNumber) {
    let accounts;
    try {
        const sql = `SELECT * FROM accounts WHERE cycleNumber BETWEEN ? AND ? ORDER BY cycleNumber ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        accounts = await db.all(_1.accountDatabase, sql, [startCycleNumber, endCycleNumber]);
        if (accounts.length > 0) {
            accounts.forEach((account) => {
                if (account && account.data)
                    account.data = (0, serialization_1.DeSerializeFromJsonString)(account.data);
            });
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Account accounts between cycles', accounts ? accounts.length : accounts, 'skip', skip);
    }
    return accounts;
}
exports.queryAccountsBetweenCycles = queryAccountsBetweenCycles;
//# sourceMappingURL=accounts.js.map