"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryLatestOriginalTxs = exports.queryOriginalTxDataCountByCycles = exports.queryOriginalTxDataByTxId = exports.queryOriginalTxsData = exports.queryOriginalTxDataCount = exports.bulkInsertOriginalTxsData = exports.insertOriginalTxData = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
const Utils_1 = require("../utils/Utils");
async function insertOriginalTxData(OriginalTxData) {
    try {
        const fields = Object.keys(OriginalTxData).join(', ');
        const placeholders = Object.keys(OriginalTxData).fill('?').join(', ');
        const values = db.extractValues(OriginalTxData);
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from OriginalTxData with txId ${OriginalTxData.txId}`);
        }
        const sql = 'INSERT OR REPLACE INTO originalTxsData (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(_1.originalTxDataDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted OriginalTxData', OriginalTxData.txId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert OriginalTxData or it is already stored in to database', OriginalTxData.txId);
    }
}
exports.insertOriginalTxData = insertOriginalTxData;
async function bulkInsertOriginalTxsData(originalTxsData) {
    try {
        const fields = Object.keys(originalTxsData[0]).join(', ');
        const placeholders = Object.keys(originalTxsData[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(originalTxsData);
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from originalTxsData. Number of originalTxsData: ${originalTxsData.length}`);
        }
        let sql = 'INSERT OR REPLACE INTO originalTxsData (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < originalTxsData.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(_1.originalTxDataDatabase, sql, values);
        Logger.mainLogger.debug('Successfully inserted OriginalTxsData', originalTxsData.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert OriginalTxsData', originalTxsData.length);
        throw e; // check with Achal/Jai
    }
}
exports.bulkInsertOriginalTxsData = bulkInsertOriginalTxsData;
async function queryOriginalTxDataCount(startCycle, endCycle) {
    let originalTxsData;
    try {
        let sql = `SELECT COUNT(*) FROM originalTxsData`;
        const values = [];
        if (startCycle && endCycle) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        originalTxsData = await db.get(_1.originalTxDataDatabase, sql, values);
    }
    catch (e) {
        console.log(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData count', originalTxsData);
    }
    return originalTxsData['COUNT(*)'] || 0;
}
exports.queryOriginalTxDataCount = queryOriginalTxDataCount;
async function queryOriginalTxsData(skip = 0, limit = 10, startCycle, endCycle) {
    let originalTxsData;
    try {
        let sql = `SELECT * FROM originalTxsData`;
        const sqlSuffix = ` ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        const values = [];
        if ((0, Utils_1.isNumber)(startCycle) && (0, Utils_1.isNumber)(endCycle)) {
            sql += ` WHERE cycle BETWEEN ? AND ?`;
            values.push(startCycle, endCycle);
        }
        sql += sqlSuffix;
        originalTxsData = await db.all(_1.originalTxDataDatabase, sql, values);
        originalTxsData.forEach((originalTxData) => {
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(originalTxData.originalTxData);
        });
    }
    catch (e) {
        console.log(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData originalTxsData', originalTxsData);
    }
    return originalTxsData;
}
exports.queryOriginalTxsData = queryOriginalTxsData;
async function queryOriginalTxDataByTxId(txId) {
    try {
        const sql = `SELECT * FROM originalTxsData WHERE txId=?`;
        const originalTxData = (await db.get(_1.originalTxDataDatabase, sql, [txId]));
        if (originalTxData) {
            if (originalTxData.originalTxData)
                originalTxData.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(originalTxData.originalTxData);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('OriginalTxData txId', originalTxData);
        }
        return originalTxData;
    }
    catch (e) {
        console.log(e);
    }
    return null;
}
exports.queryOriginalTxDataByTxId = queryOriginalTxDataByTxId;
async function queryOriginalTxDataCountByCycles(start, end) {
    let originalTxsData;
    try {
        const sql = `SELECT cycle, COUNT(*) FROM originalTxsData GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
        originalTxsData = await db.all(_1.originalTxDataDatabase, sql, [start, end]);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('OriginalTxData count by cycle', originalTxsData);
    }
    if (originalTxsData.length > 0) {
        originalTxsData.forEach((OriginalTxData) => {
            OriginalTxData['originalTxsData'] = OriginalTxData['COUNT(*)'];
            delete OriginalTxData['COUNT(*)'];
        });
    }
    return originalTxsData;
}
exports.queryOriginalTxDataCountByCycles = queryOriginalTxDataCountByCycles;
async function queryLatestOriginalTxs(count) {
    try {
        const sql = `SELECT * FROM originalTxsData ORDER BY cycle DESC, timestamp DESC LIMIT ${count ? count : 100}`;
        const originalTxs = (await db.all(_1.originalTxDataDatabase, sql));
        if (originalTxs.length > 0) {
            originalTxs.forEach((tx) => {
                if (tx.originalTxData)
                    tx.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(tx.originalTxData);
            });
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Latest Original-Tx: ', originalTxs);
        }
        return originalTxs;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryLatestOriginalTxs = queryLatestOriginalTxs;
//# sourceMappingURL=originalTxsData.js.map