"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryReceiptsBetweenCycles = exports.queryReceiptCountBetweenCycles = exports.queryReceiptCountByCycles = exports.queryReceiptCount = exports.queryReceipts = exports.queryLatestReceipts = exports.queryReceiptByReceiptId = exports.bulkInsertReceipts = exports.insertReceipt = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
async function insertReceipt(receipt) {
    try {
        const fields = Object.keys(receipt).join(', ');
        const placeholders = Object.keys(receipt).fill('?').join(', ');
        const values = db.extractValues(receipt);
        if (!values) {
            throw new Error('Failed to extract values from receipt');
        }
        const sql = 'INSERT OR REPLACE INTO receipts (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(_1.receiptDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted Receipt', receipt.receiptId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert Receipt or it is already stored in to database', receipt.receiptId);
    }
}
exports.insertReceipt = insertReceipt;
async function bulkInsertReceipts(receipts) {
    try {
        const fields = Object.keys(receipts[0]).join(', ');
        const placeholders = Object.keys(receipts[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(receipts);
        if (!values) {
            throw new Error('Failed to extract values from receipt');
        }
        let sql = 'INSERT OR REPLACE INTO receipts (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < receipts.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(_1.receiptDatabase, sql, values);
        Logger.mainLogger.debug('Successfully inserted Receipts', receipts.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert Receipts', receipts.length);
    }
}
exports.bulkInsertReceipts = bulkInsertReceipts;
async function queryReceiptByReceiptId(receiptId, timestamp = 0) {
    try {
        const sql = `SELECT * FROM receipts WHERE receiptId=?` + (timestamp ? ` AND timestamp=?` : '');
        const value = timestamp ? [receiptId, timestamp] : [receiptId];
        const receipt = (await db.get(_1.receiptDatabase, sql, value));
        if (receipt)
            deserializeDBReceipt(receipt);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Receipt receiptId', receipt);
        }
        return receipt;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryReceiptByReceiptId = queryReceiptByReceiptId;
async function queryLatestReceipts(count) {
    try {
        const sql = `SELECT * FROM receipts ORDER BY cycle DESC, timestamp DESC LIMIT ${count ? count : 100}`;
        const receipts = (await db.all(_1.receiptDatabase, sql));
        if (receipts.length > 0) {
            receipts.forEach((receipt) => {
                deserializeDBReceipt(receipt);
            });
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Receipt latest', receipts);
        }
        return receipts;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryLatestReceipts = queryLatestReceipts;
async function queryReceipts(skip = 0, limit = 10000) {
    let receipts = [];
    try {
        const sql = `SELECT * FROM receipts ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        receipts = (await db.all(_1.receiptDatabase, sql));
        if (receipts.length > 0) {
            receipts.forEach((receipt) => {
                deserializeDBReceipt(receipt);
            });
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Receipt receipts', receipts ? receipts.length : receipts, 'skip', skip);
    }
    return receipts;
}
exports.queryReceipts = queryReceipts;
async function queryReceiptCount() {
    let receipts;
    try {
        const sql = `SELECT COUNT(*) FROM receipts`;
        receipts = await db.get(_1.receiptDatabase, sql, []);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Receipt count', receipts);
    }
    if (receipts)
        receipts = receipts['COUNT(*)'];
    else
        receipts = 0;
    return receipts;
}
exports.queryReceiptCount = queryReceiptCount;
async function queryReceiptCountByCycles(start, end) {
    let receipts;
    try {
        const sql = `SELECT cycle, COUNT(*) FROM receipts GROUP BY cycle HAVING cycle BETWEEN ? AND ? ORDER BY cycle ASC`;
        receipts = await db.all(_1.receiptDatabase, sql, [start, end]);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Receipt count by cycle', receipts);
    }
    if (receipts.length > 0) {
        receipts.forEach((receipt) => {
            receipt['receipts'] = receipt['COUNT(*)'];
            delete receipt['COUNT(*)'];
        });
    }
    return receipts;
}
exports.queryReceiptCountByCycles = queryReceiptCountByCycles;
async function queryReceiptCountBetweenCycles(startCycleNumber, endCycleNumber) {
    let receipts;
    try {
        const sql = `SELECT COUNT(*) FROM receipts WHERE cycle BETWEEN ? AND ?`;
        receipts = await db.get(_1.receiptDatabase, sql, [startCycleNumber, endCycleNumber]);
    }
    catch (e) {
        console.log(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Receipt count between cycles', receipts);
    }
    if (receipts)
        receipts = receipts['COUNT(*)'];
    else
        receipts = 0;
    return receipts;
}
exports.queryReceiptCountBetweenCycles = queryReceiptCountBetweenCycles;
async function queryReceiptsBetweenCycles(skip = 0, limit = 10000, startCycleNumber, endCycleNumber) {
    let receipts = [];
    try {
        const sql = `SELECT * FROM receipts WHERE cycle BETWEEN ? AND ? ORDER BY cycle ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        receipts = (await db.all(_1.receiptDatabase, sql, [startCycleNumber, endCycleNumber]));
        if (receipts.length > 0) {
            receipts.forEach((receipt) => {
                deserializeDBReceipt(receipt);
            });
        }
    }
    catch (e) {
        console.log(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Receipt receipts between cycles', receipts ? receipts.length : receipts, 'skip', skip);
    }
    return receipts;
}
exports.queryReceiptsBetweenCycles = queryReceiptsBetweenCycles;
function deserializeDBReceipt(receipt) {
    receipt.tx &&= (0, serialization_1.DeSerializeFromJsonString)(receipt.tx);
    receipt.afterStates &&= (0, serialization_1.DeSerializeFromJsonString)(receipt.afterStates);
    receipt.beforeStates &&= (0, serialization_1.DeSerializeFromJsonString)(receipt.beforeStates);
    receipt.signedReceipt &&= (0, serialization_1.DeSerializeFromJsonString)(receipt.signedReceipt);
    receipt.appReceiptData &&= (0, serialization_1.DeSerializeFromJsonString)(receipt.appReceiptData);
    // globalModification is stored as 0 or 1 in the database, convert it to boolean
    receipt.globalModification = receipt.globalModification === 1;
}
//# sourceMappingURL=receipts.js.map