"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryTransactionsBetweenCycles = exports.queryTransactionCountBetweenCycles = exports.queryTransactionCount = exports.queryTransactions = exports.queryLatestTransactions = exports.queryTransactionByAccountId = exports.queryTransactionByTxId = exports.bulkInsertTransactions = exports.insertTransaction = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
async function insertTransaction(transaction) {
    try {
        const fields = Object.keys(transaction).join(', ');
        const placeholders = Object.keys(transaction).fill('?').join(', ');
        const values = db.extractValues(transaction); // Ensure values is always an array
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from transaction with txId ${transaction.txId}`);
        }
        const sql = 'INSERT OR REPLACE INTO transactions (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(_1.transactionDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted Transaction', transaction.txId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert Transaction or it is already stored in to database', transaction.txId);
    }
}
exports.insertTransaction = insertTransaction;
async function bulkInsertTransactions(transactions) {
    try {
        const fields = Object.keys(transactions[0]).join(', ');
        const placeholders = Object.keys(transactions[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(transactions) || [];
        if (!values || values.length === 0) {
            throw new Error(`No values extracted from transactions. Number of transactions: ${transactions.length}`);
        }
        let sql = 'INSERT OR REPLACE INTO transactions (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < transactions.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(_1.transactionDatabase, sql, values);
        Logger.mainLogger.debug('Successfully inserted Transactions', transactions.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert Transactions', transactions.length);
    }
}
exports.bulkInsertTransactions = bulkInsertTransactions;
async function queryTransactionByTxId(txId) {
    try {
        const sql = `SELECT * FROM transactions WHERE txId=?`;
        const transaction = (await db.get(_1.transactionDatabase, sql, [txId]));
        if (transaction) {
            if (transaction.data)
                transaction.data = (0, serialization_1.DeSerializeFromJsonString)(transaction.data);
            if (transaction.originalTxData)
                transaction.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(transaction.originalTxData);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Transaction txId', transaction);
        }
        return transaction;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryTransactionByTxId = queryTransactionByTxId;
async function queryTransactionByAccountId(accountId) {
    try {
        const sql = `SELECT * FROM transactions WHERE accountId=?`;
        const transaction = (await db.get(_1.transactionDatabase, sql, [accountId]));
        if (transaction) {
            if (transaction.data)
                transaction.data = (0, serialization_1.DeSerializeFromJsonString)(transaction.data);
            if (transaction.originalTxData)
                transaction.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(transaction.originalTxData);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Transaction accountId', transaction);
        }
        return transaction;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryTransactionByAccountId = queryTransactionByAccountId;
async function queryLatestTransactions(count) {
    try {
        const sql = `SELECT * FROM transactions ORDER BY cycleNumber DESC, timestamp DESC LIMIT ${count ? count : 100}`;
        const transactions = (await db.all(_1.transactionDatabase, sql));
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if (transaction.data)
                    transaction.data = (0, serialization_1.DeSerializeFromJsonString)(transaction.data);
                if (transaction.originalTxData)
                    transaction.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(transaction.originalTxData);
            });
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Transaction latest', transactions);
        }
        return transactions;
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
}
exports.queryLatestTransactions = queryLatestTransactions;
async function queryTransactions(skip = 0, limit = 10000) {
    let transactions;
    try {
        const sql = `SELECT * FROM transactions ORDER BY cycleNumber ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        transactions = await db.all(_1.transactionDatabase, sql);
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if (transaction.data)
                    transaction.data = (0, serialization_1.DeSerializeFromJsonString)(transaction.data);
                if (transaction.originalTxData)
                    transaction.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(transaction.originalTxData);
            });
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Transaction transactions', transactions ? transactions.length : transactions, 'skip', skip);
    }
    return transactions;
}
exports.queryTransactions = queryTransactions;
async function queryTransactionCount() {
    let transactions;
    try {
        const sql = `SELECT COUNT(*) FROM transactions`;
        transactions = await db.get(_1.transactionDatabase, sql, []);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Transaction count', transactions);
    }
    if (transactions)
        transactions = transactions['COUNT(*)'];
    else
        transactions = 0;
    return transactions;
}
exports.queryTransactionCount = queryTransactionCount;
async function queryTransactionCountBetweenCycles(startCycleNumber, endCycleNumber) {
    let transactions;
    try {
        const sql = `SELECT COUNT(*) FROM transactions WHERE cycleNumber BETWEEN ? AND ?`;
        transactions = await db.get(_1.transactionDatabase, sql, [startCycleNumber, endCycleNumber]);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Transaction count between cycles', transactions);
    }
    if (transactions)
        transactions = transactions['COUNT(*)'];
    else
        transactions = 0;
    return transactions;
}
exports.queryTransactionCountBetweenCycles = queryTransactionCountBetweenCycles;
async function queryTransactionsBetweenCycles(skip = 0, limit = 10000, startCycleNumber, endCycleNumber) {
    let transactions;
    try {
        const sql = `SELECT * FROM transactions WHERE cycleNumber BETWEEN ? AND ? ORDER BY cycleNumber ASC, timestamp ASC LIMIT ${limit} OFFSET ${skip}`;
        transactions = await db.all(_1.transactionDatabase, sql, [startCycleNumber, endCycleNumber]);
        if (transactions.length > 0) {
            transactions.forEach((transaction) => {
                if (transaction.data)
                    transaction.data = (0, serialization_1.DeSerializeFromJsonString)(transaction.data);
                if (transaction.originalTxData)
                    transaction.originalTxData = (0, serialization_1.DeSerializeFromJsonString)(transaction.originalTxData);
            });
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Transaction transactions between cycles', transactions ? transactions.length : transactions, 'skip', skip);
    }
    return transactions;
}
exports.queryTransactionsBetweenCycles = queryTransactionsBetweenCycles;
//# sourceMappingURL=transactions.js.map