"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const v8 = require("v8");
const path_1 = require("path");
const Logger = require("./Logger");
const dbstore = require("./dbstore");
const clusterModule = require("cluster");
const child_process_1 = require("./child-process");
const utils_1 = require("./utils");
const Config_1 = require("./Config");
const utils_2 = require("./distributor/utils");
const rmq_data_publisher_1 = require("./distributor/rmq_data_publisher");
const checkpoints_1 = require("./dbstore/checkpoints");
const rmq_healthcheck_server_1 = require("./distributor/rmq_healthcheck_server");
const rmq_healthcheck_service_1 = require("./distributor/rmq_healthcheck_service");
const metrics_1 = require("./metrics");
const cluster = clusterModule;
// Override default config params from config file, env vars, and cli args
const file = (0, path_1.join)(process.cwd(), 'distributor-config.json');
const { argv, env } = process;
const spawnWorker = () => {
    const worker = cluster.fork();
    utils_2.workerClientMap.set(worker, []);
    utils_2.workerProcessMap.set(worker.process.pid, worker);
    (0, utils_2.registerWorkerMessageListener)(worker);
    if (Config_1.config.limitToSubscribersOnly)
        (0, utils_2.refreshSubscribers)();
    return worker;
};
const initDistributor = async () => {
    // Common logic for both parent and worker processes
    (0, Config_1.overrideDefaultConfig)(file, env, argv);
    // Set crypto hash keys from config
    const hashKey = Config_1.config.DISTRIBUTOR_HASH_KEY;
    (0, utils_1.setHashKey)(hashKey);
    (0, utils_1.initLogger)();
    addSigListeners();
    (0, utils_2.updateConfigAndSubscriberList)();
    if (cluster.isPrimary) {
        // Primary/Parent Process Logic
        Logger.mainLogger.debug(`Distributor Master Process (${process.pid}) Started`);
        for (let i = 0; i < Config_1.config.NUMBER_OF_WORKERS; i++) {
            // const worker: Worker = cluster.fork({
            //   execArgv: process.execArgv.concat([`--inspect=0.0.0.0:${9229 + i}`])
            // });
            const worker = spawnWorker();
            console.log(`⛏️ Worker ${worker.process.pid} started`);
        }
        cluster.on('exit', (worker, code, signal) => {
            Logger.mainLogger.debug(`❌ Worker ${worker.process.pid} exited with code ${code} and signal ${signal}`);
            const outGoingWorker = utils_2.workerProcessMap.get(worker.process.pid);
            if (outGoingWorker) {
                utils_2.workerClientMap.delete(outGoingWorker);
                utils_2.workerProcessMap.delete(outGoingWorker.process.pid);
            }
            const newWorker = spawnWorker();
            Logger.mainLogger.debug(`⛏️ Worker ${newWorker.process.pid} started to replace the terminated one`);
        });
        console.log(`Primary Process Heap Memory limit: ${v8.getHeapStatistics().heap_size_limit / 1024 / 1024} MB`);
        setInterval(() => {
            const memoryUsage = process.memoryUsage();
            console.log(`Primary Process Heap Used: ${memoryUsage.heapUsed / 1024 / 1024} MB / ${memoryUsage.heapTotal / 1024 / 1024} MB`);
        }, 30000); // log every 30 seconds
    }
    else {
        if (Config_1.config.distributorMode == Config_1.distributorMode.WS) {
            await (0, child_process_1.initWorker)();
        }
        // Worker Process Logic
        await dbstore.initializeDB(Config_1.config);
        await (0, metrics_1.setInitialDataLogReaderMetrics)();
        const { worker } = cluster;
        await (0, child_process_1.initHttpServer)(worker);
        console.log(`Worker ${worker.process.pid} > Heap Memory limit: ${v8.getHeapStatistics().heap_size_limit / 24 / 1024} MB`);
        setInterval(() => {
            const memoryUsage = process.memoryUsage();
            console.log(`Heap Used in Worker ${worker.process.pid}: ${memoryUsage.heapUsed / 1024 / 1024} MB / ${memoryUsage.heapTotal / 1024 / 1024} MB`);
        }, 30000); // log every 30 seconds
    }
};
const addSigListeners = () => {
    process.on('SIGUSR1', async () => {
        // Reload the distributor-config.json
        (0, Config_1.overrideDefaultConfig)(file, env, argv);
        Logger.mainLogger.debug('Config reloaded', Config_1.config);
        Logger.mainLogger.debug('DETECTED SIGUSR1 SIGNAL @: ', process.pid);
        console.log('DETECTED SIGUSR1 SIGNAL @: ', process.pid);
        if (cluster.isPrimary) {
            // Check for expired subscribers in the updated config
            if (Config_1.config.limitToSubscribersOnly)
                (0, utils_2.refreshSubscribers)();
        }
        else {
            // Refresh the list of subscribers in every worker process
            (0, utils_2.updateConfigAndSubscriberList)();
        }
    });
    process.on('SIGINT', async () => {
        console.log('Exiting on SIGINT');
        if (!cluster.isPrimary)
            await dbstore.closeDatabases();
        process.exit(0);
    });
    process.on('SIGTERM', async () => {
        console.log('Exiting on SIGTERM');
        if (!cluster.isPrimary)
            await dbstore.closeDatabases();
        process.exit(0);
    });
    process.on('uncaughtException', (error) => {
        console.error('Uncaught Exception in Distributor: ', error);
    });
    Logger.mainLogger.debug('Registered signal listeners.');
};
const initDistributorMQMode = async () => {
    const rmqDataPublisher = new rmq_data_publisher_1.default();
    (0, utils_1.initLogger)();
    await dbstore.initializeDB(Config_1.config);
    await checkpoints_1.CheckpointDao.init();
    addSigListeners();
    (0, utils_2.updateConfigAndSubscriberList)();
    const rmqHealthCheck = new rmq_healthcheck_service_1.default(rmqDataPublisher);
    await (0, rmq_healthcheck_server_1.initRMQModeHttpServer)(rmqHealthCheck);
    rmqDataPublisher.start();
    process.on('SIGTERM', async () => {
        console.log('Exiting initDistributorMQMode on SIGTERM');
        await rmqDataPublisher.cleanUp();
        checkpoints_1.CheckpointDao.close();
        process.exit(0);
    });
    process.on('SIGINT', async () => {
        console.log('Exiting initDistributorMQMode on SIGTERM');
        await rmqDataPublisher.cleanUp();
        checkpoints_1.CheckpointDao.close();
        process.exit(0);
    });
};
if (Config_1.config.distributorMode === Config_1.distributorMode.MQ) {
    console.log(`Initiating Distributor in MQ mode`);
    initDistributorMQMode();
}
else {
    console.log(`Initiating Distributor in WS mode`);
    initDistributor();
}
//# sourceMappingURL=distributor.js.map