"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const conn_1 = require("../messaging/rabbitmq/conn");
const Utils_1 = require("../utils/Utils");
const cycles_1 = require("../dbstore/cycles");
const receipts_1 = require("../dbstore/receipts");
const checkpoints_1 = require("../dbstore/checkpoints");
const Config_1 = require("../Config");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Crypto = require("../utils/Crypto");
const originalTxsData_1 = require("../dbstore/originalTxsData");
class TxData {
    constructor(txId, txTs) {
        this.txId = txId;
        this.txTs = txTs;
    }
}
class ReceiptData {
    constructor(receiptId, receiptTs) {
        this.receiptId = receiptId;
        this.receiptTs = receiptTs;
    }
}
class RMQDataPublisher {
    constructor() {
        this.isConnected = false;
        this.isConnClosing = false;
        this.cycleCursor = 0;
        this.batchSize = 5;
        this.cursorUpdatedAt = Date.now();
        this.cycleConfirmThreshold = this.batchSize - 1;
        this.cursorUpdateThresholdInMillis = 60000; // 1 minute
        this.checkpointDataType = 'cycle';
        this.cycleVsTxMap = new Map();
        this.cycleVsRcptMap = new Map();
        this.cyclePublishedMap = new Map();
        this.ttlInMillis = 24 * 60 * 60 * 1000; // 1 day
        this.handleConnectionClose = async () => {
            console.error(`[RMQDataPublisher#handleConnectionClose]: Connection closed}`);
            if (this.isConnClosing === true) {
                // this is triggered internally, possibly on SIGTERM/SIGINT; so no need to retry
                return Promise.resolve();
            }
            this.isConnected = false;
            return new Promise((resolve) => {
                this.retryConnection();
                resolve();
            });
        };
    }
    async start() {
        await this.connect();
        this.startInMemoryMapCleanupJob();
        // get cursor from db
        const checkpoint = await checkpoints_1.CheckpointDao.getCheckpoint(this.checkpointDataType, Config_1.distributorMode.MQ);
        if (checkpoint != null && checkpoint.id != null && checkpoint.id > 0) {
            this.cycleCursor = parseInt(checkpoint.cursor);
        }
        const process = true;
        while (process) {
            console.log(`[RMQDataPublisher] Started job to publish events to RMQ. isConnClosing: ${this.isConnClosing} | isConnected: ${this.isConnected}`);
            if (this.isConnClosing) {
                if (this.cleanUpJobInterval != null) {
                    clearInterval(this.cleanUpJobInterval);
                }
                return;
            }
            if (!this.isConnected) {
                await (0, Utils_1.sleep)(500);
                continue;
            }
            try {
                await this.findAndPublishEvents();
                console.log(`[RMQDataPublisher] Competed job to publish events to RMQ`);
            }
            catch (e) {
                console.log(`❌ [RMQDataPublisher#start]: Error in publishing distributor events ${e}`);
            }
            await (0, Utils_1.sleep)(200);
        }
    }
    getCursorUpdatedAt() {
        return this.cursorUpdatedAt;
    }
    async findAndPublishEvents() {
        const start = this.cycleCursor;
        const end = this.cycleCursor + this.batchSize - 1;
        const cyclesFromDB = (await (0, cycles_1.queryCycleRecordsBetween)(start, end)) || [];
        if (cyclesFromDB.length == 0) {
            console.log(`No cycles found for publishing: start: ${start} | end: ${end}`);
            this.cycleCursor = end; // if we don't set this, we'll end up stuck at a particular cursor
            return;
        }
        console.log(`Got cycles from DB: ${cyclesFromDB.length} between cycles ${start} and ${end}`);
        const cycles = [];
        for (const cycle of cyclesFromDB) {
            const txMap = this.cycleVsTxMap.get(cycle.counter);
            if (txMap === null || txMap === undefined) {
                this.cycleVsTxMap.set(cycle.counter, new Map());
            }
            const rcptMap = this.cycleVsRcptMap.get(cycle.counter);
            if (rcptMap === null || rcptMap === undefined) {
                this.cycleVsRcptMap.set(cycle.counter, new Map());
            }
            if (!(this.cyclePublishedMap.get(cycle.counter) === true)) {
                cycles.push(cycle);
            }
        }
        const transactions = [];
        const receipts = [];
        const fetch = true;
        const limit = 1000;
        let skip = 0;
        while (fetch) {
            const txns = (await (0, originalTxsData_1.queryOriginalTxsData)(skip, limit, start, end));
            console.log(`Got txns: ${txns.length} between cycles ${start} and ${end}`);
            // check for new or updated transactions
            for (const txn of txns) {
                let txMap = this.cycleVsTxMap.get(txn.cycle);
                if (txMap === null || txMap === undefined) {
                    this.cycleVsTxMap.set(txn.cycle, new Map());
                    txMap = this.cycleVsTxMap.get(txn.cycle);
                }
                const txData = txMap.get(txn.txId);
                if (txData === null || txData === undefined) {
                    txMap.set(txn.txId, new TxData(txn.txId, txn.timestamp));
                    transactions.push(txn);
                }
                else if (txData.txTs < txn.timestamp) {
                    transactions.push(txn);
                }
            }
            if (txns.length < limit) {
                break;
            }
            skip += txns.length;
        }
        skip = 0;
        while (fetch) {
            const rcpts = await (0, receipts_1.queryReceiptsBetweenCycles)(skip, limit, start, end);
            console.log(`Got receipts: ${rcpts.length} between cycles ${start} and ${end}`);
            // check for new or updated receipts
            for (const receipt of rcpts) {
                let receiptMap = this.cycleVsRcptMap.get(receipt.cycle);
                if (receiptMap === null || receiptMap === undefined) {
                    this.cycleVsRcptMap.set(receipt.cycle, new Map());
                    receiptMap = this.cycleVsRcptMap.get(receipt.cycle);
                }
                const receiptData = receiptMap.get(receipt.receiptId);
                if (receiptData === null || receiptData === undefined) {
                    receiptMap.set(receipt.receiptId, new ReceiptData(receipt.receiptId, receipt.timestamp));
                    receipts.push(receipt);
                }
                else if (receiptData.receiptTs < receipt.timestamp) {
                    receipts.push(receipt);
                }
            }
            if (rcpts.length < limit) {
                break;
            }
            skip += rcpts.length;
        }
        console.log(`Publishing cycles: ${cycles.length}`);
        console.log(`Publishing transactions: ${transactions.length}`);
        console.log(`Publishing receipts: ${receipts.length}`);
        const promises = [];
        promises.push(this.publishCycles(cycles));
        promises.push(this.publishTransactions(transactions));
        promises.push(this.publishReceipts(receipts));
        await Promise.all(promises);
        // we wait for this.cycleConfirmThreshold cycles, before we move to cursor to next cycle
        // note that this.batchSize for fetching cycles should always be greater than 3 or we will never fetch new cycles
        let updateCursor = false;
        if (cyclesFromDB.length >= this.cycleConfirmThreshold) {
            updateCursor = true;
            this.cycleCursor = cyclesFromDB[cyclesFromDB.length - this.cycleConfirmThreshold].counter;
        }
        // update cursor
        // we won't update cursor everytime
        const diff = Date.now() - this.cursorUpdatedAt;
        if (updateCursor && diff >= this.cursorUpdateThresholdInMillis) {
            this.cursorUpdatedAt = Date.now();
            checkpoints_1.CheckpointDao.upsertCheckpoint(this.checkpointDataType, Config_1.distributorMode.MQ, JSON.stringify(this.cycleCursor));
        }
    }
    async publishCycles(cycles) {
        if (cycles.length <= 0)
            return;
        const messages = [];
        for (let i = 0; i < cycles.length; i++) {
            const cycle = cycles.at(i);
            messages.push({
                cycle: {
                    counter: cycle.counter,
                    cycleMarker: cycle.marker,
                    cycleRecord: cycle,
                },
            });
        }
        await this.publishMessages(process.env.RMQ_CYCLES_EXCHANGE_NAME, messages);
        for (let i = 0; i < cycles.length; i++) {
            const cycle = cycles.at(i);
            this.cyclePublishedMap.set(cycle.counter, true);
        }
    }
    async publishTransactions(transactions) {
        if (transactions.length <= 0)
            return;
        const messages = [];
        for (let i = 0; i < transactions.length; i++) {
            messages.push({
                originalTx: transactions.at(i),
            });
        }
        await this.publishMessages(process.env.RMQ_ORIGINAL_TXS_EXCHANGE_NAME, messages);
        for (let i = 0; i < transactions.length; i++) {
            const txn = transactions.at(i);
            const txMap = this.cycleVsTxMap.get(txn.cycle);
            txMap.set(txn.txId, new TxData(txn.txId, txn.timestamp));
        }
    }
    async publishReceipts(receipts) {
        if (receipts.length <= 0)
            return;
        const messages = [];
        for (let i = 0; i < receipts.length; i++) {
            messages.push({
                receipt: receipts.at(i),
            });
        }
        await this.publishMessages(process.env.RMQ_RECEIPTS_EXCHANGE_NAME, messages);
        for (let i = 0; i < receipts.length; i++) {
            const receipt = receipts.at(i);
            const receiptMap = this.cycleVsRcptMap.get(receipt.cycle);
            receiptMap.set(receipt.receiptId, new ReceiptData(receipt.receiptId, receipt.timestamp));
        }
    }
    async publishMessages(exchange, messages) {
        let retry = 3;
        let lastErr = null;
        while (retry > 0) {
            retry--;
            try {
                for (let i = 0; i < messages.length; i++) {
                    const message = {
                        signedData: Crypto.sign(messages.at(i), Config_1.config.DISTRIBUTOR_SECRET_KEY, Config_1.config.DISTRIBUTOR_PUBLIC_KEY),
                    };
                    if (!this.isConnected) {
                        return;
                    }
                    this.channel.publish(exchange, '', Buffer.from(lib_types_1.Utils.safeStringify(message.signedData)), {
                        persistent: true,
                        expiration: this.ttlInMillis,
                    });
                    // console.log(`published message on ${exchange}: ${msgStr}`)
                }
                const start = Date.now();
                await this.channel.waitForConfirms();
                const end = Date.now();
                console.log(`Done waiting for confirmation for messages on ${exchange} | Time taken: ${end - start} ms`);
                lastErr = null;
                return;
            }
            catch (e) {
                lastErr = e;
                console.log(`❌ [${exchange.toUpperCase()} publishMessages] Error while publishing message to queue: ${e}`);
            }
        }
        if (lastErr != null) {
            throw lastErr;
        }
    }
    async connect() {
        try {
            this.conn = await new conn_1.default('data_publisher').getConnection();
            this.conn.on('close', this.handleConnectionClose);
            this.conn.on('error', this.handleConnectionError);
            this.channel = await this.conn.createConfirmChannel();
            this.channel.on('error', this.handleConnectionError);
            this.channel.on('close', this.handleConnectionClose);
            this.isConnected = true;
            console.log(`✅ [RMQDataPublisher#connect]: Successfully connected to RabbitMQ`);
        }
        catch (e) {
            console.log(`❌ [RMQDataPublisher#connect]: error while connecting to RabbitMQ: ${e}`);
            throw e;
        }
    }
    handleConnectionError(error) {
        console.error(`[RMQDataPublisher#handleConnectionError]: Connection error: ${error}`);
        this.isConnected = false;
    }
    retryConnection() {
        let attempt = 0;
        if (!this.isConnected) {
            const interval = setInterval(async () => {
                attempt++;
                console.log(`[retryConnection]: (Attempt ${attempt}) intitiated connection retry...`);
                try {
                    await this.connect();
                    console.log(`[retryConnection]: (Attempt ${attempt}) successfully connected...`);
                    this.isConnected = true;
                    clearInterval(interval);
                }
                catch (e) {
                    console.log(`[retryConnection]: (Attempt ${attempt}) unsuccessul. Err: ${e}`);
                }
            }, 5000); // Wait 5 seconds before retrying
        }
    }
    async cleanUp() {
        this.isConnClosing = true;
        // sleep for some time to allow on going execution
        (0, Utils_1.sleep)(2000);
        if (this.channel != null) {
            await this.channel.close();
        }
        if (this.conn != null) {
            await this.conn.close();
        }
    }
    async startInMemoryMapCleanupJob() {
        this.cleanUpJobInterval = setInterval(() => {
            console.log(`Started in-memory clean up job`);
            for (const key of this.cyclePublishedMap.keys()) {
                if (key < this.cycleCursor - this.cycleConfirmThreshold) {
                    console.log(`Deleted cycle for key from memory: ${key}`);
                    this.cyclePublishedMap.delete(key);
                }
            }
            for (const key of this.cycleVsTxMap.keys()) {
                if (key < this.cycleCursor - this.cycleConfirmThreshold) {
                    console.log(`Deleted txns for key from memory: ${key}`);
                    this.cycleVsTxMap.delete(key);
                }
            }
            for (const key of this.cycleVsRcptMap.keys()) {
                if (key < this.cycleCursor - this.cycleConfirmThreshold) {
                    console.log(`Deleted receipts for key from memory: ${key}`);
                    this.cycleVsRcptMap.delete(key);
                }
            }
            console.log(`Completed in-memory clean up job`);
        }, 60 * 1000 // 1 minute
        );
    }
}
exports.default = RMQDataPublisher;
//# sourceMappingURL=rmq_data_publisher.js.map