"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWorkerForClient = exports.refreshSubscribers = exports.updateConfigAndSubscriberList = exports.registerWorkerMessageListener = exports.showAllProcesses = exports.socketServer = exports.distributorSubscribers = exports.workerProcessMap = exports.socketClientMap = exports.workerClientMap = void 0;
const Logger = require("../Logger");
const Config_1 = require("../Config");
let subscriberRefreshInterval = null;
exports.workerClientMap = new Map();
exports.socketClientMap = new Map();
exports.workerProcessMap = new Map();
exports.distributorSubscribers = new Map();
const showAllProcesses = () => {
    const clients = [];
    for (const [key, value] of exports.workerClientMap.entries()) {
        clients.push({ Worker_PID: key.process.pid, Client_PubKeys: value });
    }
    console.table(clients, ['Worker_PID', 'Client_PubKeys']);
};
exports.showAllProcesses = showAllProcesses;
const registerWorkerMessageListener = (worker) => {
    worker.on('message', ({ type, data }) => {
        switch (type) {
            case 'client_close':
                Logger.mainLogger.debug('❌ Client Connection Terminated with ID: ', data.toString());
                removeSocketClient(data.toString());
                break;
            case 'client_connected':
                {
                    if (isClientAlreadyConnected(data.toString())) {
                        const workerForExistingClient = (0, exports.getWorkerForClient)(data.toString());
                        if (workerForExistingClient.process.pid !== worker.process.pid) {
                            Logger.mainLogger.debug(`❌ Closing previous connection of Client (${data}) with worker ${workerForExistingClient.process.pid}`);
                            worker?.send({ type: 'close_duplicate_connection', data });
                            removeSocketClient(data.toString());
                        }
                        else {
                            // Note: If the client is already connected to the same worker, then the worker will terminate the previous connection on its own.
                            Logger.mainLogger.debug(`❌ Duplicate Connection Request of Client (${data}) on the same worker ${workerForExistingClient.process.pid}`);
                            removeSocketClient(data.toString());
                        }
                    }
                    Logger.mainLogger.debug(`✅ Client (${data}) connected with Worker: ${worker.process.pid}`);
                    exports.socketClientMap.set(data.toString(), worker.process.pid);
                    const clients = exports.workerClientMap.get(worker);
                    clients.push(data.toString());
                    exports.workerClientMap.set(worker, clients);
                    (0, exports.showAllProcesses)();
                }
                break;
            case 'client_expired':
                Logger.mainLogger.debug('❌ Client with ID Expired: ', data.toString());
                removeSocketClient(data.toString());
                break;
            default:
                if (type)
                    Logger.mainLogger.debug('Unexpected Message Received From Worker: ', { type, data });
        }
    });
};
exports.registerWorkerMessageListener = registerWorkerMessageListener;
const isClientAlreadyConnected = (clientId) => {
    return exports.socketClientMap.has(clientId);
};
const updateConfigAndSubscriberList = () => {
    const subscribers = Config_1.config.subscribers;
    // Clear the previous subscribers list
    exports.distributorSubscribers.clear();
    // Load the new subscribers list from the config
    for (let i = 0; i < subscribers.length; i++) {
        exports.distributorSubscribers.set(subscribers[i].publicKey, subscribers[i]);
    }
    Logger.mainLogger.debug('Subscribers refreshed, count: ', exports.distributorSubscribers.size);
};
exports.updateConfigAndSubscriberList = updateConfigAndSubscriberList;
const refreshSubscribers = () => {
    if (subscriberRefreshInterval) {
        clearInterval(subscriberRefreshInterval);
    }
    const subscribers = Config_1.config.subscribers;
    subscriberRefreshInterval = setInterval(() => {
        for (let i = 0; i < subscribers.length; i++) {
            // Subscribers with expirationTimestamp of 0 are permanent subscribers
            if (subscribers[i].expirationTimestamp !== 0 && subscribers[i].expirationTimestamp < Date.now()) {
                Logger.mainLogger.debug(`❌ Removing Expired Subscriber: ${subscribers[i].publicKey}`);
                const childProcess = (0, exports.getWorkerForClient)(subscribers[i].publicKey);
                childProcess?.send({ type: 'remove_subscriber', data: subscribers[i].publicKey });
                subscribers.splice(i, 1);
            }
        }
    }, 30000);
};
exports.refreshSubscribers = refreshSubscribers;
const removeSocketClient = (clientId) => {
    console.log('Removing Client: ', clientId, exports.socketClientMap.has(clientId));
    exports.socketClientMap.delete(clientId);
    for (const [workerProcess, clients] of exports.workerClientMap.entries()) {
        const index = clients.findIndex((id) => id === clientId);
        if (index > -1) {
            clients.splice(index, 1);
            exports.workerClientMap.set(workerProcess, clients);
            console.log(`Client (${clientId}) disconnected from Child Process (${workerProcess.process.pid})`);
            (0, exports.showAllProcesses)();
            return;
        }
    }
};
const getWorkerForClient = (clientId) => {
    const workerProcessId = exports.socketClientMap.get(clientId);
    if (!workerProcessId)
        throw new Error(`Inside: ${process.pid} Child process associated with Client: ${clientId} not found.`);
    const workerProcess = exports.workerProcessMap.get(workerProcessId);
    if (!workerProcess)
        throw new Error(`Child process with PID: ${workerProcessId} not found.`);
    return workerProcess;
};
exports.getWorkerForClient = getWorkerForClient;
//# sourceMappingURL=utils.js.map