"use strict";
/**
 * Implmentation of a log reader that reads data from log files written by the collector
 * service and emits the same to the socket clients connected to their respective child processes.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable security/detect-non-literal-fs-filename */
const fs = require("fs");
const path = require("path");
const EventEmitter = require("events");
const readline = require("readline");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Config_1 = require("../Config");
class DataLogReader extends EventEmitter {
    constructor(logDir, dataName) {
        super();
        this.logDir = logDir;
        this.dataName = dataName;
        this.logCounter = 1;
        this.activeLogFileName = `active-${dataName}-log.txt`;
    }
    async init() {
        const activeLogFilePath = path.join(this.logDir, this.activeLogFileName);
        if (await this.fileExists(activeLogFilePath)) {
            // Read the active log file to get the current log counter
            const activeLog = fs.readFileSync(activeLogFilePath, {
                encoding: 'utf8',
            });
            const activeLogNumber = parseInt(activeLog.replace(`${this.dataName}-log`, '').replace('.txt', ''));
            console.log(`Active log file found: ${activeLogFilePath} with number ${activeLogNumber}`);
            if (activeLogNumber) {
                this.logCounter = activeLogNumber;
            }
            // Read the log file and skip the entries that have already been recorded
            const logFile = path.join(this.logDir, `${this.dataName}-log${this.logCounter}.txt`);
            if (!(await this.fileExists(logFile))) {
                throw new Error(`Log file ${logFile} does not exist`);
            }
            const stat = fs.statSync(logFile);
            let startEntries = 0;
            let foundEndNumberofEntriesLine = false;
            const stream = fs.createReadStream(logFile, {
                encoding: 'utf8',
                start: 0,
                end: stat.size,
            });
            const rl = readline.createInterface({
                input: stream,
            });
            rl.on('line', (data) => {
                /* prettier-ignore */ if (Config_1.config.VERBOSE)
                    console.log(data);
                try {
                    // if the data contains a line with "End: Number of entries: ", then we know that the file has been rotated.
                    if (data.includes('End: Number of entries: ')) {
                        foundEndNumberofEntriesLine = true;
                    }
                    else {
                        startEntries++;
                    }
                    /* prettier-ignore */ if (Config_1.config.VERBOSE)
                        console.log(this.dataName, 'startEntries', startEntries);
                }
                catch (e) {
                    /* prettier-ignore */ if (Config_1.config.VERBOSE)
                        console.log('data is not complete!');
                }
            });
            stream.on('end', () => {
                console.log('Finished reading the file.');
                if (!foundEndNumberofEntriesLine) {
                    console.log(`Starting at ${this.dataName}-log${this.logCounter}.txt with size ${stat.size} and ${startEntries} entries`);
                    this.readLogFile(stat.size, startEntries);
                }
                rl.close();
                stream.close();
            });
        }
        // listen to the active log file
        fs.watch(activeLogFilePath, async (event, filename) => {
            if (filename) {
                let activeLog = '';
                do {
                    activeLog = fs.readFileSync(activeLogFilePath, {
                        encoding: 'utf8',
                    });
                } while (activeLog === '');
                const activeLogNumber = parseInt(activeLog.replace(`${this.dataName}-log`, '').replace('.txt', ''));
                if (isNaN(activeLogNumber))
                    return;
                if (activeLogNumber === this.logCounter)
                    return;
                if (activeLogNumber) {
                    this.logCounter = activeLogNumber;
                }
                console.info(`Switching to new File ${this.dataName}-log${this.logCounter}.txt`);
                this.readLogFile();
            }
        });
    }
    async readLogFile(startSize = 0, startEntries = 0) {
        const logFile = path.join(this.logDir, `${this.dataName}-log${this.logCounter}.txt`);
        if (!(await this.fileExists(logFile))) {
            throw new Error(`Log file ${logFile} does not exist`);
        }
        let currentSize = startSize;
        let totalNumberOfEntries = startEntries;
        let reading = false;
        const fileStreamer = setInterval(() => {
            if (reading)
                return;
            reading = true;
            fs.stat(logFile, (err, stats) => {
                if (err) {
                    console.error(err);
                    reading = false;
                    return;
                }
                if (stats.size > currentSize) {
                    if (Config_1.config.VERBOSE)
                        console.log(this.dataName, 'totalNumberOfEntries', totalNumberOfEntries, 'currentSize', currentSize, 'stats.size', stats.size);
                    const sizeDiff = stats.size - currentSize;
                    if (sizeDiff / 1024 / 1024 > 100)
                        console.log(this.dataName, stats.size, currentSize, sizeDiff / 1024 / 1024, 'MB');
                    const stream = fs.createReadStream(logFile, {
                        encoding: 'utf8',
                        start: currentSize,
                        end: stats.size,
                    });
                    const rl = readline.createInterface({
                        input: stream,
                    });
                    // read the stream line-by-line
                    rl.on('line', (data) => {
                        try {
                            // if the data contains a line with "End: Number of entries: ", then we know that the file has been rotated.
                            if (data.includes('End: Number of entries: ')) {
                                const entryCount = parseInt(data.split(':').slice(-1)[0].trim());
                                if (isNaN(entryCount)) {
                                    console.error('Error: File has been rotated but the last line does not contain the total number of entries');
                                }
                                if (totalNumberOfEntries !== entryCount)
                                    console.log('Total number of entries does not not match with the entry Count.', totalNumberOfEntries, entryCount);
                                this.emit(`${this.dataName}-end`, totalNumberOfEntries, entryCount);
                                // End the interval
                                clearInterval(fileStreamer);
                            }
                            else {
                                const parse = lib_types_1.Utils.safeJsonParse(data);
                                totalNumberOfEntries += 1;
                                /* prettier-ignore */ if (Config_1.config.VERBOSE)
                                    console.log(`${this.dataName}-data`, data);
                                this.emit(`${this.dataName}-data`, parse);
                            }
                            currentSize += Buffer.byteLength(data + '\n');
                        }
                        catch (e) {
                            console.error(this.dataName, '❌ Damaged line Detected! >: ', data.length, data);
                            if (Config_1.config.VERBOSE)
                                console.log(this.dataName, 'Damaged line Detected! >: ', 'totalNumberOfEntries', totalNumberOfEntries, 'currentSize', currentSize);
                        }
                    });
                    rl.on('error', (err) => {
                        console.error('Error occurred in File Streamer', logFile, err);
                    });
                    stream.on('end', () => {
                        // End of line
                        rl.close();
                        stream.close();
                        reading = false;
                    });
                }
                else {
                    reading = false;
                }
            });
        }, Config_1.config.FILE_STREAM_INTERVAL_MS); // Check for new data
    }
    async fileExists(filePath) {
        try {
            await fs.promises.access(filePath, fs.constants.F_OK);
            return true;
        }
        catch (err) {
            return false;
        }
    }
}
exports.default = DataLogReader;
//# sourceMappingURL=index.js.map