"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initRMQAPIServer = void 0;
const path_1 = require("path");
const dbstore = require("./dbstore");
const Config_1 = require("./Config");
const cors_1 = require("@fastify/cors");
const fastify_1 = require("fastify");
const rate_limit_1 = require("@fastify/rate-limit");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const api_1 = require("./api");
const utils_1 = require("./utils");
const Logger = require("./Logger");
const utils_2 = require("./distributor/utils");
const file = (0, path_1.join)(process.cwd(), 'distributor-config.json');
const { argv, env } = process;
const addSigListeners = () => {
    process.on('SIGUSR1', async () => {
        // Reload the distributor-config.json
        (0, Config_1.overrideDefaultConfig)(file, env, argv);
        Logger.mainLogger.debug('Config reloaded', Config_1.config);
        Logger.mainLogger.debug('DETECTED SIGUSR1 SIGNAL @: ', process.pid);
        console.log('DETECTED SIGUSR1 SIGNAL @: ', process.pid);
        (0, utils_2.updateConfigAndSubscriberList)();
    });
    process.on('SIGINT', async () => {
        console.log('Exiting on SIGINT');
        await dbstore.closeDatabases();
        process.exit(0);
    });
    process.on('SIGTERM', async () => {
        console.log('Exiting on SIGTERM');
        await dbstore.closeDatabases();
        process.exit(0);
    });
    process.on('uncaughtException', (error) => {
        console.error('Uncaught Exception in Distributor: ', error);
    });
    Logger.mainLogger.debug('Registered signal listeners.');
};
const initRMQAPIServer = async () => {
    (0, Config_1.overrideDefaultConfig)(file, env, argv);
    (0, utils_1.setHashKey)(Config_1.config.DISTRIBUTOR_HASH_KEY);
    (0, utils_1.initLogger)();
    addSigListeners();
    (0, utils_2.updateConfigAndSubscriberList)();
    await dbstore.initializeDB(Config_1.config);
    const fastifyServer = (0, fastify_1.default)({
        logger: false,
    });
    await fastifyServer.register(cors_1.default);
    await fastifyServer.register(rate_limit_1.default, {
        global: true,
        max: Config_1.config.RATE_LIMIT,
        timeWindow: 10,
        allowList: ['127.0.0.1', '0.0.0.0'], // Excludes local IPs from rate limits
    });
    fastifyServer.addContentTypeParser('application/json', { parseAs: 'string' }, (req, body, done) => {
        try {
            const jsonString = typeof body === 'string' ? body : body.toString('utf8');
            done(null, lib_types_1.Utils.safeJsonParse(jsonString));
        }
        catch (err) {
            err.statusCode = 400;
            done(err, undefined);
        }
    });
    fastifyServer.setReplySerializer((payload) => {
        return lib_types_1.Utils.safeStringify(payload);
    });
    // Register API routes
    (0, api_1.registerRoutes)(fastifyServer);
    // Start server and bind to port on all interfaces
    fastifyServer.listen({
        port: Config_1.config.DISTRIBUTOR_PORT,
        host: '0.0.0.0',
    }, (err) => {
        Logger.mainLogger.debug('MQ Distributor-Server listening on port', Config_1.config.DISTRIBUTOR_PORT);
        if (err) {
            fastifyServer.log.error(err);
            process.exit(1);
        }
        Logger.mainLogger.debug('MQ Distributor-Server started');
    });
};
exports.initRMQAPIServer = initRMQAPIServer;
(0, exports.initRMQAPIServer)();
//# sourceMappingURL=rmq_api_server.js.map