"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const checkpoints_1 = require("../../../src/dbstore/checkpoints");
const cycles_1 = require("../../../src/dbstore/cycles");
const rmq_data_publisher_1 = require("../../../src/distributor/rmq_data_publisher");
const Utils_1 = require("../../../src/utils/Utils");
const amqp = require("amqplib");
const originalTxsData_1 = require("../../../src/dbstore/originalTxsData");
const receipts_1 = require("../../../src/dbstore/receipts");
const Crypto = require("../../../src/utils/Crypto");
const Config_1 = require("../../../src/Config");
jest.mock('amqplib', () => ({
    connect: jest.fn(),
}));
jest.mock('../../../src/dbstore/checkpoints', () => ({
    CheckpointDao: {
        getCheckpoint: jest.fn(),
        upsertCheckpoint: jest.fn(),
    },
}));
jest.mock('../../../src/dbstore/cycles', () => ({
    queryCycleRecordsBetween: jest.fn(),
}));
jest.mock('../../../src/dbstore/receipts', () => ({
    queryReceiptsBetweenCycles: jest.fn(),
}));
jest.mock('../../../src/dbstore/originalTxsData', () => ({
    queryOriginalTxsData: jest.fn(),
}));
jest.mock('../../../src/utils/Utils', () => ({
    sleep: jest.fn(),
}));
jest.mock('../../../src/utils/Crypto', () => ({
    sign: jest.fn(),
}));
describe('RMQDataPublisher Tests', () => {
    let mockConnect;
    beforeAll(() => {
        mockConnect = amqp.connect;
    });
    let publisher;
    beforeEach(() => {
        publisher = new rmq_data_publisher_1.default();
        // jest.clearAllMocks()
    });
    test('should initialize and connect', async () => {
        ;
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([]);
        Utils_1.sleep.mockResolvedValue(undefined);
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        const startPromise = publisher.start();
        // Simulate the loop
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(checkpoints_1.CheckpointDao.getCheckpoint).toHaveBeenCalledWith('cycle', Config_1.distributorMode.MQ);
    });
    test('should handle connection errors and retry', async () => {
        const connectSpy = jest.spyOn(publisher, 'connect').mockResolvedValue({});
        const retryConnectionSpy = jest.spyOn(publisher, 'retryConnection').mockImplementation();
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([]);
        Utils_1.sleep.mockResolvedValue(undefined);
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        const startPromise = publisher.start();
        // Simulate the loop
        await (0, Utils_1.sleep)(1000);
        publisher.handleConnectionClose();
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(connectSpy).toHaveBeenCalled();
        expect(retryConnectionSpy).toHaveBeenCalled();
    });
    test('should publish events when connected', async () => {
        ;
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        checkpoints_1.CheckpointDao.upsertCheckpoint.mockResolvedValue({});
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([{ counter: 1 }, { counter: 2 }]);
        originalTxsData_1.queryOriginalTxsData.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        receipts_1.queryReceiptsBetweenCycles.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        Utils_1.sleep.mockResolvedValue(undefined);
        Crypto.sign.mockResolvedValue({});
        const connectSpy = jest.spyOn(publisher, 'connect').mockResolvedValue({});
        const findAndPublishEventsSpy = jest.spyOn(publisher, 'findAndPublishEvents');
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
            publish: jest.fn(),
            waitForConfirms: jest.fn().mockResolvedValue(true),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        mockChannel.publish.mockResolvedValue('true');
        publisher.channel = mockChannel;
        const startPromise = publisher.start();
        await (0, Utils_1.sleep)(1000);
        publisher.isConnected = true;
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(connectSpy).toHaveBeenCalled();
        expect(findAndPublishEventsSpy).toHaveBeenCalled();
    });
    test('should update checkpoint after given threshold', async () => {
        ;
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        checkpoints_1.CheckpointDao.upsertCheckpoint.mockResolvedValue({});
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([
            { counter: 1 },
            { counter: 2 },
            { counter: 3 },
            { counter: 4 },
            { counter: 5 },
            { counter: 6 },
        ]);
        originalTxsData_1.queryOriginalTxsData.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        receipts_1.queryReceiptsBetweenCycles.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        Utils_1.sleep.mockResolvedValue(undefined);
        Crypto.sign.mockResolvedValue({});
        const connectSpy = jest.spyOn(publisher, 'connect').mockResolvedValue({});
        const findAndPublishEventsSpy = jest.spyOn(publisher, 'findAndPublishEvents');
        const upsertCheckpointSpy = jest.spyOn(checkpoints_1.CheckpointDao, 'upsertCheckpoint');
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
            publish: jest.fn(),
            waitForConfirms: jest.fn().mockResolvedValue(true),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        mockChannel.publish.mockResolvedValue('true');
        publisher.channel = mockChannel;
        publisher.cursorUpdatedAt = Date.now() - 61000;
        const startPromise = publisher.start();
        await (0, Utils_1.sleep)(1000);
        publisher.isConnected = true;
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(connectSpy).toHaveBeenCalled();
        expect(findAndPublishEventsSpy).toHaveBeenCalled();
        expect(upsertCheckpointSpy).toHaveBeenCalled();
    });
    test('should safely return on no new events', async () => {
        ;
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        checkpoints_1.CheckpointDao.upsertCheckpoint.mockResolvedValue({});
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([]);
        originalTxsData_1.queryOriginalTxsData.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        receipts_1.queryReceiptsBetweenCycles.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        Utils_1.sleep.mockResolvedValue(undefined);
        Crypto.sign.mockResolvedValue({});
        const connectSpy = jest.spyOn(publisher, 'connect').mockResolvedValue({});
        const findAndPublishEventsSpy = jest.spyOn(publisher, 'findAndPublishEvents');
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
            publish: jest.fn(),
            waitForConfirms: jest.fn().mockResolvedValue(true),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        mockChannel.publish.mockResolvedValue('true');
        publisher.channel = mockChannel;
        const startPromise = publisher.start();
        await (0, Utils_1.sleep)(1000);
        publisher.isConnected = true;
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(connectSpy).toHaveBeenCalled();
        expect(findAndPublishEventsSpy).toHaveBeenCalled();
        expect(mockChannel.publish).not.toHaveBeenCalled();
    });
    test('should not invoke publish method if connection is closed', async () => {
        ;
        checkpoints_1.CheckpointDao.getCheckpoint.mockResolvedValue({ id: 1, cursor: '0' });
        checkpoints_1.CheckpointDao.upsertCheckpoint.mockResolvedValue({});
        cycles_1.queryCycleRecordsBetween.mockResolvedValue([{ counter: 1 }, { counter: 2 }]);
        originalTxsData_1.queryOriginalTxsData.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        receipts_1.queryReceiptsBetweenCycles.mockResolvedValue([{ cycle: 1 }, { cycle: 2 }]);
        Utils_1.sleep.mockResolvedValue(undefined);
        Crypto.sign.mockResolvedValue({});
        const connectSpy = jest.spyOn(publisher, 'connect').mockResolvedValue({});
        const findAndPublishEventsSpy = jest.spyOn(publisher, 'findAndPublishEvents');
        const mockChannel = {
            on: jest.fn(),
            close: jest.fn(),
            publish: jest.fn(),
            waitForConfirms: jest.fn().mockResolvedValue(true),
        };
        const mockConnection = {
            createConfirmChannel: jest.fn().mockResolvedValue(mockChannel),
            close: jest.fn(),
            on: jest.fn(),
        };
        mockConnect.mockResolvedValue(mockConnection);
        mockChannel.publish.mockResolvedValue('true');
        publisher.channel = mockChannel;
        const startPromise = publisher.start();
        await (0, Utils_1.sleep)(1000);
        publisher.isConnected = true;
        await (0, Utils_1.sleep)(1000);
        publisher.isConnected = false;
        await (0, Utils_1.sleep)(1000);
        publisher.isConnClosing = true;
        await startPromise;
        expect(connectSpy).toHaveBeenCalled();
        expect(findAndPublishEventsSpy).toHaveBeenCalled();
        expect(mockChannel.publish).not.toHaveBeenCalled();
    });
});
//# sourceMappingURL=rmq_data_publisher.test.js.map